/* analyzer.c -- analyze protocol communication data.  
   Copyright (C) 2004 Petrozavodsk State University.

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
   USA.  */

#include <stdio.h>
#include <stdlib.h>

#include "config.h"
#include "analyzer.h"
#include "cmdlparse.h"
#include "logread.h"
#include "msgmap.h"
#include "eventcalc.h"
#include "pefwrite.h"
#include "error.h"
#include "util.h"


/* The name of the output file.  */
char *output_file_name = PEF_DEFAULT_NAME;

/* The name of variables log file.  */
char *variables_log_name = NULL;

/* List of host structures.  */
struct host *host_list = NULL;

/* Number of packet trace files. */
int packet_trace_file_count;

/* List of links structures.  */
struct link *link_list = NULL;

/* List of flow structures.  */
struct flow *flow_list = NULL;

/* Application layer ports.  */
struct app_ports ports = {
  {HTTP_PORT_DEFAULT}, 1,
  {DNS_PORT_DEFAULT}, 1
};

/* Exists ARP-layer flag.  */
short int arp_exist = 0;


int
main (int argc, char **argv)
{
  struct ptu **ptu_lists;
  struct event *es = NULL;
  struct event *current_event;
  struct ptu *current_ptu;
  struct host *current_host;
  struct flow *current_flow;
  struct link *current_link;
  struct ptu *ptp;
  int i;

  /* Calls CMD line parser module.  */
  parse_args (argc, argv);

  /* Make initial data structures.  */
  ptu_lists = malloc_check (sizeof (struct ptu *) * packet_trace_file_count);
  current_host = host_list;

  /* Calls Log reader module.  */
  for (i = 0; i < packet_trace_file_count; i++)
    {
      read_log (current_host, &ptu_lists[i]);
      current_host = current_host->next;
    }

  /* Calls Message mapper module.  */
  ptp = map_messages (ptu_lists);

  /* Delete unused and make initial data structures.  */
  free (ptu_lists);

  /* Calls Event calculator module.  */
  calculate (ptp, &es, &link_list, &flow_list, variables_log_name);

  /* Calls PEF writer module.  */
  pef_writef (output_file_name, es, host_list, link_list, flow_list);


  /* Delete data structures.  */
  current_event = es;		/* delete events list.  */
  while (current_event != NULL)
    {
      es = current_event->next;
#ifdef DEBUG
      fprintf (stderr, "id=%p\n", (void *) current_event->id);
#endif
      if (current_event->unit_next != NULL && current_event->id != NULL)
	{
#ifdef DEBUG
	  fprintf (stderr, "next id=%p\n",
		   (void *) current_event->unit_next->id);
#endif
	  current_event->unit_next->id = NULL;
	}
      if (current_event->unit_prev != NULL && current_event->id != NULL)
	{
#ifdef DEBUG
	  fprintf (stderr, "prev id=%p\n",
		   (void *) current_event->unit_prev->id);
#endif
	  current_event->unit_prev->id = NULL;
	}
      free (current_event->id);
      current_event->id = NULL;
      free (current_event);
      current_event = es;
    }
  current_ptu = ptp;		/* delete ptu list.  */
  while (current_ptu != NULL)
    {
      ptp = current_ptu->next;
      free (current_ptu->net_hdr);
      free (current_ptu->trans_hdr);
      free (current_ptu->app_data);
      free (current_ptu);
      current_ptu = ptp;
    }
  /* delete host_list structures.  */
  current_host = host_list;
  while (current_host != NULL)
    {
      host_list = host_list->next;
      free (current_host->id);
      free (current_host);
      current_host = host_list;
    }
  /* delete link_list structures.  */
  current_link = link_list;
  while (current_link != NULL)
    {
      link_list = link_list->next;
      free (current_link->id);
      free (current_link);
      current_link = link_list;
    }
  /* delete flow_list structures.  */
  current_flow = flow_list;
  while (current_flow != NULL)
    {
      flow_list = flow_list->next;
      free (current_flow->id);
      free (current_flow);
      current_flow = flow_list;
    }

  /* Exit successfully.  */
  return (EXIT_SUCCESS);
}
