/* ec_common.h -- declarations for common event calculator functions.
   Copyright (C) 2004 Petrozavodsk State University.

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
   USA.  */

/* Author: Mikhail Kryshen
   Date:   04.05.2004 */

#ifndef EC_COMMON_H
#define EC_COMMON_H

enum
{
  FRAG_IMMEDIATE = 0,
  FRAG_FOLLOWS = 1,
  FRAG_NONSEQ = -1
};

struct protocol_module
{
  /* True if module supports restoring of child events.  */
  int can_restore;

  /* Functions used for defragmentation (restoring).  */
  int (*is_fragment_of) (struct event *event, struct event *fragment);
  int (*is_first_fragment) (struct event *event, struct ptu *source);
  int (*compare_fragments) (struct event *f1, struct event *f2);

  /* Not yet unfragmented events (linked by next pointer).  */
  struct event *restore_first;
  struct event *restore_last;
  
  /* Not yet processed fragments (linked by next_parent pointer).  */
  struct event *unprocessed_first;
};

extern struct protocol_module protocol_modules[];

/*
 * Initialize protocol modules.  
 */
void protocol_modules_init ();

/*
 * Set default variables
 */
void set_default_vars (struct event *event);

/* 
 * Insert event to the end of the linked list after the specified element.
 */
void insert_event (struct event **first, struct event **last,
		   struct event *after, struct event *event);

/* 
 * Add event to the end of the linked list.
 */
void add_event (struct event **first, struct event **last, 
		struct event *event);

/*
 * Remove event from the linked list.  
 */
void remove_event (struct event **first, struct event **last, 
		   struct event *event);

/* 
 * Add event to the end of the linked list of parents.  
 */
void add_parent_event (struct event **first, struct event **last,
		       struct event *event);

/*
 * Remove event from the linked list of parents.  
 */
void
remove_parent_event (struct event **first, struct event *prev,
		     struct event *event);

#endif /* EC_COMMON_H */
