/* ec_udp.c -- UDP support for event splitter.
   Copyright (C) 2004 Petrozavodsk State University.

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
   USA.  */

/* Author: Mikhail Kryshen
   Date:   04.05.2004 */

#include "analyzer.h"
#include "ec_common.h"

/*
 * Check for the UDP fragment of a application-level event.  
 */
int
is_udp_fragment_of (struct event *event, struct event *fragment)
{
  if (fragment->protocol != PTU_UDP ||
      event->parent == NULL || event->parent->protocol != PTU_UDP)
    {
      return 0;
    }

  return (event->parent == fragment);
}

int
is_first_udp_fragment (struct event *event, struct ptu *source)
{
  return 1;
}

/*
 * Check relation of 2 fragments.
 *
 * Returns FRAG_IMMEDIATE if fragment f2 immedeatly follows f1,
 * FRAG_FOLLOWS if fragment f2 follws f1 with some fragments between them,
 * FRAG_NONSEQ otherwise.  
 */
int
compare_udp_fragments (struct event *f1, struct event *f2)
{
  if (f2 == NULL)
    return FRAG_IMMEDIATE;
  else
    return FRAG_NONSEQ;
}

void 
udp_module_init (struct protocol_module *module)
{
  module->can_restore = 1;
  module->is_fragment_of = &is_udp_fragment_of;
  module->is_first_fragment = &is_first_udp_fragment;
  module->compare_fragments = &compare_udp_fragments;
}
