/* error.h -- declarations for error processing module.
   Copyright (C) 2004 Petrozavodsk State University.

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
   USA.  */

/* Author: Andrew Salo
   Date:   12.04.2004 */

#ifndef ERROR_H
#define ERROR_H

/* Symbolic error names.  */
enum
{
  ERR_NONE = 0,			/* No error.  */
  ERR_UNKNOWN = 1,		/* Unknown error.  */
  ERR_IP = 2,			/* Invalid IP address.  */
  ERR_NUM_IP = 3,		/* Invalid number of IP addresses.  */
  ERR_NUM_FILES = 4,		/* Invalid number of packet trace files.  */
  ERR_NUM_TIMEALIGN = 5,	/* Invalid number of time alignments.  */
  ERR_TIMEALIGN = 6,		/* Invalid time alignment value.  */
  ERR_NUM_PORTS = 7,		/* Invalid number of ports.  */
  ERR_PORT = 8,			/* Invalid port value.  */
  ERR_DLT_NOTSUP = 9,		/* Unsupported data link type.  */
  ERR_TRUNC_PKT = 10,		/* Truncated packet.  */
  ERR_MM_TIMESTAMP = 11,	/* Packet sent time is greater that the received time.  */
  ERR_UNKNOWN_ADDRESS = 12,	/* Could not determine source or destination address.  */
  WARN_IRRELEVANT_UNIT = 13,	/* Unit was not sent or received at the processing host.  */
  WARN_EQ_TIMESTAMPS = 14,	/* Events have the same timestamps.  */
  WARN_NOT_SENT = 15,           /* Received unit that was not sent.  */
  WARN_VAR_LOGS_NOT_IMPL = 16   /* Variables logs are not supported.  */
};

/* Print error message and terminate the program.  */
void error (int status, const char *cause, const char *descr);

/* Print warning.  */
void warning (const char *cause, const char *descr);

/* Get descriptive error message string.  */
const char *anlz_strerror (int err);

#endif /* ERROR_H */
