/* util.h -- declarations for common utility functions.
   Copyright (C) 2004 Petrozavodsk State University.

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
   USA.  */

#ifndef UTIL_H
#define UTIL_H

#include <netinet/in.h>
#include <net/if_arp.h>

/* Choose minimum and maximum from two values.  */
#define MIN(a,b) (((a) < (b)) ? (a) : (b))
#define MAX(a,b) (((a) < (b)) ? (b) : (a))

/* Convert time from struct timeval to double.  */
#define TVTOD(tv) ((double) (tv).tv_sec + ((double) (tv).tv_usec) / 1000000)

/* Allocates memory checking for errors.  */
void *malloc_check (int len);

/* Allocates memory and copies data checking for errors.  */
void *copy_data (void *src, int len);


/* Gets source and destination IP addresses from
   network layer protocol header.  */
int get_ip_addrs (void *hdr, int type, struct in_addr *src,
		  struct in_addr *dst);

/* Extracts sender and target hardware addresses from ARP header.  */
int get_arp_hrd_addrs (struct arphdr *hdr, char *snd, char *trg, size_t cnt);

/* Compare time as structure timeval.
   Return 0 if times equal; -1 if first time smaller; +1 if first time biger  */
int cmp_timevals (struct timeval time1, struct timeval time2);

/* Addition time as structure timeval.  */
struct timeval add_timeval (struct timeval time1, struct timeval time2);

/* Time difference (second minus first).  */
struct timeval diff_timeval (struct timeval time1, struct timeval time2);

/* Convert ip address as long int to string format.  */
char *ip2str (u_int32_t ip_i);

/* Defragment the event data.  */
int defragment_data (struct event *event, char *buffer);

/* Compare strings in HTTP message header.  */
const char *strhttpstr (const char *haystack, const char *needle);

#endif /* UTIL_H */
